@@ -297,12 +297,12 @@ static snd_pcm_sframes_t snd_pcm_us_write(snd_pcm_ioplug_t *io,
 	return size;
 }
 
-static int usb_stream_read(struct user_usb_stream *uus, void *to, unsigned bytes)
+static int usb_stream_read(struct user_usb_stream *uus, void *to)
 {
 	struct usb_stream *s = uus->s;
-	int p = s->inpacket_split, l = 0;
+	unsigned p = s->inpacket_split, l = 0;
 	void *i = (void *)s + s->inpacket[p].offset + s->inpacket_split_at;
-	int il = s->inpacket[p].length - s->inpacket_split_at;
+	unsigned il = s->inpacket[p].length - s->inpacket_split_at;
 
 	do {
 		if (l + il > s->period_size)
@@ -337,8 +337,7 @@ static snd_pcm_sframes_t snd_pcm_us_read(snd_pcm_ioplug_t *io,
 			return -EINVAL;
 		}
 		if (us->uus->s->periods_done - us->periods_done == 1) {
-			red = usb_stream_read(us->uus, to, size * frame_size) /
-				frame_size;
+			red = usb_stream_read(us->uus, to) / frame_size;
 			us->periods_done++;
 			return red;
 		}
@@ -424,8 +423,10 @@ static int snd_pcm_us_open(snd_pcm_t **pcmp, const char *name,
 	}
 	VDBG("%i %s", stream, us_name);
 	us->uus = get_uus(card);
-	if (!us->uus)
+	if (!us->uus) {
+		free(us);
 		return -ENOMEM;
+	}
 	err = snd_hwdep_open(&us->hwdep, us_name, O_RDWR);
 	if (err < 0) {
 		us_free(us);
@@ -455,7 +456,6 @@ static int snd_pcm_us_open(snd_pcm_t **pcmp, const char *name,
 	err = us_set_hw_constraint(us);
 	if (err < 0) {
 		snd_pcm_ioplug_delete(&us->io);
-		us_free(us);
 		return err;
 	}
 
